==> mupad-1.4.2.tgz: mupad/share/mmg/include/kernel/MIO_local.h <==
/* $Date: 1998/03/02 09:51:39 $  $Author: kg $  $Revision: 1.30.2.2 $ */

/**************************************************************************/
/* FILE:             MIO_local.h                                          */
/* Autor:            Oliver Kluge, Andreas Kemper, Ralf Hillebrand        */ 
/* Erstellt:         18/08/92                                             */ 
/* Letzte Aenderung: 07/08/94                                             */ 
/**************************************************************************/ 
 
#ifndef __MIO_local__
#define __MIO_local__


/**************************************************************************/ 
/* Dieses File enthaelt die internen Routinen der Ausgabe .               */  
/**************************************************************************/ 
 
/* ein leerer Ausgabe-Knoten */
#define MIONULL ((MIOV_PRETTY_PRINT)?   \
                MIO_out_make_text(0L,0L,0L,MIOC_P_max,"")\
		:MMMNULL)

/* Produkt-Zeichen fuer den PRETTY_PRINTer */
#define MIOC_MULT_STRING (MEVV_pref.printTimesDot ? "\xb7" : " ")

/* MIOC_DONT_SPLIT-Bit auf special-Komponente ausblenden */ 
inline short
MIO_bitmask(short spec) {return(spec & ~MIOC_DONT_SPLIT);}

/* Wie MDB_make_debug_info, nur ist der Schluessel bereits bekannt und 
   nicht der Filename */
#define MIO_make_debug_info( line, key, valid )   \
	( line << SHIFT | key << 1 | valid )

#define MIO_create_info( line )		\
	MDB_create_debug_info( MIO_make_debug_info( line, MIOV_KEY, 1) )

/* Macros zur Erstellung eines tempor"aren Feldes von 'num'-char-Werten */

#define MIO_new_string(l, sp, num)                                    \
	{ sp = MMMmalloc(MMMCglobal,MMM_TMP,sizeof(char)*((num)+1), 0L);\
          *MMMarr(&sp,(num),char,MMMwrite) = '\0';                      \
	  l = MMMZ(&sp,char,MMMwrite);                                \
	  MMMpoolinsert(&l); }


#define MIO_free_string(l, sp)                                        \
        { MMMpooldelete(&l);					      \
	  MMMfree(sp); }

/* Berechnung des Maximums zweier Zahlen            */

#define MIO_max(a,b) (((a)<(b))?(b):(a))

#ifdef MDX
#  define MIOC_RESTORE 0L
#  define MIOC_SET  1L
#  define MIOC_RESET 2L

   void 
   MIO_change_global(long);
#endif

/****************** Funktionen in den Dateien MIO_*.c **********************/



/**** nun folgen die Ausgabe-Funktionen fuer die Kategorien *****/

S_Pointer 
MIO_out_cat_array(S_Pointer, long);

S_Pointer 
MIO_out_cat_bool(S_Pointer, long);

#ifdef MDX
S_Pointer 
MIO_out_cat_debug(S_Pointer,long);
#endif

S_Pointer 
MIO_out_cat_dom(S_Pointer, long) ;

S_Pointer 
MIO_out_cat_exec (S_Pointer, long) ;

S_Pointer 
MIO_out_cat_ext(S_Pointer, long) ;

S_Pointer 
MIO_out_cat_failed(S_Pointer, long);

S_Pointer 
MIO_out_cat_funcarg(S_Pointer, long);

S_Pointer 
MIO_out_cat_func_env(S_Pointer, long);

S_Pointer 
MIO_out_cat_ident(S_Pointer, long);

S_Pointer 
MIO_out_cat_namespace(S_Pointer, long);

S_Pointer 
MIO_out_cat_nil(S_Pointer, long);

S_Pointer 
MIO_out_cat_null(S_Pointer, long);

S_Pointer 
MIO_out_cat_point(S_Pointer, long);

S_Pointer 
MIO_out_cat_poly(S_Pointer, long);

S_Pointer 
MIO_out_cat_polygon(S_Pointer, long);

S_Pointer 
MIO_out_cat_proc(S_Pointer, long);

S_Pointer 
MIO_out_cat_set(S_Pointer, long);

S_Pointer 
MIO_out_cat_stat_list(S_Pointer, long);

S_Pointer 
MIO_out_cat_string(S_Pointer, long);

S_Pointer 
MIO_out_cat_table(S_Pointer, long);

S_Pointer /* fuer alle PARI-Zahl-Typen ausser CAT_COMPLEX */
MIO_print_number(S_Pointer, long);

S_Pointer /* fuer CAT_COMPLEX */
MIO_print_complex_number(S_Pointer, long);

 
/******************** interne Hilfsroutinen   ********************/
#ifndef MACINTOSH
void 
MIO_put_char(char);
#endif 

/* Ausgabe eines Operator-Strings als MMM_STRING */
S_Pointer 
MIO_print_symbol(S_Pointer);

/* Ausgabe eines MMM_STRING incl. Expandierung von Sonderzeichen */
S_Pointer 
MIO_out_string(S_Pointer);

/* Ausgabe einer Pari-Zahl */
void 
MIO_pari_print_number(S_Pointer);

/* gibt die Argumente eines Funktionsaufrufs geklammert aus */
S_Pointer 
MIO_out_args(S_Pointer *);

/* gibt die Argumente einer CAT_EXPR aus */
S_Pointer 
MIO_out_args_expr(S_Pointer *,long,S_Pointer);

/* gibt alle Pointer eines S_Pointers als exprseq aus 
   wird in MIO_out_cat_ext und MIO_out_cat_exec verwendet */
S_Pointer 
MIO_out_all_args (S_Pointer, MUPbool) ;

/* gibt die Argumente eines _index-Ausdrucks in eckigen Klammern aus */
S_Pointer 
MIO_out_index_args(S_Pointer,long);

/* fuehrt die Evaluierung aller "print"-Methoden durch */
S_Pointer 
MIO_call_eval(S_Pointer, S_Pointer);

/* Typpruefung und Ausgabe eines S_Pointers */ 
S_Pointer 
MIO_out_help(S_Pointer *);

/* Verzweigung je nach Category */
S_Pointer 
MIO_out_category(S_Pointer *,long,S_Pointer);

/* Verzweigung und Klammerung aller Nicht-Expressions */
S_Pointer 
MIO_out_node(S_Pointer *,long,S_Pointer);

/* Verzweigung aller Expressions */
S_Pointer 
MIO_out_expr(S_Pointer *,long,S_Pointer);

/* Zuruecksetzen des Ausgabepuffers */
void 
MIO_newline(VOID);


/* BLOCK-Modus fuer Tables, Matrizen und Anweisungen */
void 
MIO_init_BLOCK(VOID);
void 
MIO_exit_BLOCK(VOID);


/* Pretty-Ausgabe spezieller Ausdruckstypen */
S_Pointer 
MIO_out_stand_op_pretty(S_Pointer, S_Pointer, long, S_Pointer);
S_Pointer 
MIO_out_plus(S_Pointer,S_Pointer,long,S_Pointer);
S_Pointer 
MIO_out_mult(S_Pointer,S_Pointer,long,S_Pointer);
S_Pointer 
MIO_out_power(S_Pointer,S_Pointer,long,S_Pointer);
S_Pointer 
MIO_out_stmtseq(S_Pointer,S_Pointer,long,S_Pointer);


/* Hilfsroutinen zur Ausgabe von Anweisungen */
short  
MIO_out_stmt(S_Pointer *,long,S_Pointer);
S_Pointer 
MIO_init_stmt(VOID);
void 
MIO_exit_stmt(VOID);
void 
MIO_print_blanks(long,MUPbool);


/* Manipulation von MuPAD-Daten fuer die Pretty-Ausgabe */
MUPbool 
MIO_is_neg(S_Pointer);
S_Pointer
MIO_negate(S_Pointer, MUPbool plusMode=_false_);

MUPbool 
MIO_has_normal_form(S_Pointer);

MUPbool 
MIO_is_inverted(S_Pointer);
S_Pointer
MIO_invert(S_Pointer);


/* Hilfsroutinen zur Erzeugung des Ausgabebaums und dessen Ausgabe */
S_Pointer 
MIO_out_make_node(MIOT_FIELDSIZE, MIOT_FIELDSIZE, MIOT_FIELDSIZE, 
		  MIOT_FIELDSIZE, long);
S_Pointer 
MIO_out_make_text(MIOT_FIELDSIZE, MIOT_FIELDSIZE, MIOT_FIELDSIZE, 
		  MIOT_FIELDSIZE, char *);
S_Pointer 
MIO_out_make_field(MIOT_FIELDSIZE);
void 
MIO_out_write_in_field(S_Pointer , MIOT_FIELDSIZE , MIOT_FIELDSIZE *);
void 
MIO_out_node_to_field(S_Pointer , MIOT_FIELDSIZE , MIOT_FIELDSIZE *);
void 
MIO_out_text_to_field(S_Pointer , MIOT_FIELDSIZE , MIOT_FIELDSIZE *);
void 
MIO_out_print_line(MIOT_FIELDSIZE);

/* Erzeugung spezieller Strukturen */
S_Pointer 
MIO_out_write_line(MIOT_FIELDSIZE);
S_Pointer 
MIO_out_vertical_line(MIOT_FIELDSIZE);
S_Pointer 
MIO_not(S_Pointer);
S_Pointer 
MIO_div(S_Pointer);
S_Pointer 
MIO_minus(S_Pointer, long, long);
S_Pointer 
MIO_bracket(S_Pointer);
S_Pointer 
MIO_out_brackets(S_Pointer, char);
S_Pointer 
MIO_open_bracket(MIOT_FIELDSIZE ,char);
S_Pointer 
MIO_close_bracket(MIOT_FIELDSIZE ,char);
S_Pointer 
MIO_out_evaluate_children(S_Pointer ,long ,long , S_Pointer); 
void 
MIO_make_STRING_BUFFER(S_Pointer);
S_Pointer 
MIO_flush_STRING_BUFFER(VOID);
void 
MIO_make_NUM_BUFFER(S_Pointer);
S_Pointer 
MIO_flush_NUM_BUFFER(VOID);
void 
MIO_open_line(VOID);
void 
MIO_close_line(VOID);
void 
MIO_make_FIELD_BUFFER(S_Pointer);
S_Pointer 
MIO_del_trailing_zeroes(S_Pointer);
S_Pointer 
MIO_miostring2mmmstring(S_Pointer);


/* Routinen zur Bearbeitung des Ausgabebaums */
void 
MIO_put_pointer(S_Pointer,S_Pointer,long);
void 
MIO_get_param(S_Pointer,MIOT_FIELDSIZE *,MIOT_FIELDSIZE *,
	      MIOT_FIELDSIZE *,MIOT_FIELDSIZE *);
void 
MIO_put_param(S_Pointer,MIOT_FIELDSIZE,MIOT_FIELDSIZE,
	      MIOT_FIELDSIZE,MIOT_FIELDSIZE spec);
MIOT_FIELDSIZE
MIO_get_priority(S_Pointer);
void
MIO_put_priority(S_Pointer, MIOT_FIELDSIZE);
void 
MIO_get_pos(S_Pointer,long,MIOT_FIELDSIZE *line,MIOT_FIELDSIZE *);
void 
MIO_put_pos(S_Pointer,long,MIOT_FIELDSIZE,MIOT_FIELDSIZE );



/* globale Variable in den Ausgabefiles  */

/* Steuerung der Zahlausgabe bei PRETTY_PRINT */
extern MUPbool MIOV_PRINT_NUMBER;

/* Steuerung der Stringausgabe bei PRETTY_PRINT */
extern MUPbool MIOV_PRINT_STRING;

/* Flag fuer die Art der Ausgabe bei PRETTY_PRINT */
extern MUPbool MIOV_PRINT_BLOCK;        

/* Flag fuer die Art der Ausgabe bei PRETTY_PRINT */
extern MUPbool MIOV_PRINT_STMT;          

/* Stringlaenge bei PRETTY_PRINT */
extern MIOT_FIELDSIZE MIOV_STRINGLEN;
       
#ifdef MDX
#  ifndef DOS_FILENAMES
#    include "MDB_debug.h"
#  else
#    include "MDB_debu.h"
#  endif

/* steuert die Erzeugung von Debug-Info's */
extern MUPbool  MIOV_DEBUG_MODE;   

/* steuert die Erzeugung von Debug-Info's */
extern MUPbool  MIOV_REALLY_DEBUG; 

/* Aktuelle Zeilennummer fuer Debug-Mode */
extern long     MIOV_LINES;        

/* Nummer der Ausgabedatei fuer Debug-Mode*/
extern long     MIOV_KEY;               

/* letzte Debug-Info */
extern DBX_TYPE MIOV_GLOBAL_DBX;	   

/* Rekursionstiefe der STMTSEQes */
extern long 	MIOV_NUM_STMTSEQ;	   

/* wir erzeugen gerade eine expr2text-Ausgabe */
extern MUPbool  MIOV_EXPR2TEXT_MODE;
#endif  /* MDX */

/* wird fuer KeepOrder benoetigt, vgl. MIO_out_plus */
extern MUPbool  MIOV_DOM_EXT_MODE;

/* Anzahl der Zeichen, die bisher in der aktuellen Zeile stehen */
extern long MIOV_ACT_LEN;


/* Routinen des Files MIO_array.c */

void 
MIO_out_array_head(long dim,long *start,long *range,long *div);
	               

S_Pointer
MIO_out_divided_array(	S_Pointer    s,
			long         dim, 
			long        *start, 
			long        *range,
			long	    *div);

S_Pointer
MIO_out_undivided_array(S_Pointer    s,
			long         dim, 
			long        *start, 
			long        *range,
			long	    *div);

S_Pointer	
MIO_out_divided_array_pretty(	S_Pointer     s,
		 		long          dim, 
	        		long         *start,
				long         *end,
		 		long 	     *div, 
		 		long         *actblock, 
				long	     *maxblock,
		 		long         *subrange,
		 		long         *offset,
		 		long         *actpos);

void	
MIO_out_divided_array_without_pretty(	S_Pointer     s,
		 		long          dim, 
	        		long         *start,
				long         *end,
		 		long 	     *div, 
		 		long         *actblock, 
				long	     *maxblock,
		 		long         *subrange,
		 		long         *offset,
		 		long         *actpos);	

S_Pointer	
MIO_out_divided_matrix(	S_Pointer     s,
		 	long          dim, 
	       		long         *start,
			long         *end,
		 	long 	     *div, 
		 	long         *actblock, 
			long	     *maxblock,
	 		long         *subrange,
		 	long         *offset,
		 	long         *actpos);


S_Pointer	
MIO_out_undivided_array_pretty(	S_Pointer     s,
		 		long          dim, 
	        		long         *start,
				long         *end,
				long  	     *div,
		 		long         *actpos);

void	
MIO_out_undivided_array_without_pretty(	S_Pointer  s,
		 		long          dim, 
	        		long         *start,
				long         *end,
				long  	     *div,
		 		long         *actpos);

S_Pointer	
MIO_out_undivided_matrix(	S_Pointer     s,
		 		long          dim, 
	        		long         *start,
				long         *end,
		 		long         *actpos);

void	
MIO_out_subarray(S_Pointer     list,
		 long          dim, 
	         long         *start,
		 long         *end,
		 long 	      *div, 
		 long         *actblock, 
		 long         *subrange,
		 long         *offset,
		 long         *actpos);		
 	
#endif /* __MIO_local__ */

==> mupad-2.0.0.tgz: mupad/share/mmg/include/kernel/MIO_local.h <==
/* $Date: 2000/07/06 15:30:32 $  $Author: tonner $  $Revision: 1.27 $ */

/**************************************************************************/
/* FILE:             MIO_local.h                                          */
/* Autor:            Oliver Kluge, Andreas Kemper, Ralf Hillebrand        */
/* Erstellt:         18/08/92                                             */
/* Letzte Aenderung: 07/08/94                                             */
/**************************************************************************/

#ifndef __MIO_local__
#define __MIO_local__


/**************************************************************************/
/* Dieses File enthaelt die internen Routinen der Ausgabe .               */
/**************************************************************************/

/* ein leerer Ausgabe-Knoten */
#define MIONULL ((MIOV_PRETTY_PRINT)?   \
                MIO_out_make_text(0,0,0,MIOC_P_max,"")\
		:MMMNULL)

/* Newline-Ausgabe-Knoten:  Bisherigen Teilbaum zentriert ausgeben */
#define MIO_CENTERED_NEWLINE()  ((MIOV_PRETTY_PRINT)?   \
                MIO_out_make_text(-1,0,0,MIOC_P_max,"")\
		:MMMNULL)

/* Newline-Ausgabe-Knoten:  Bisherigen Teilbaum unzentriert ausgeben */
#define MIO_UNCENTERED_NEWLINE()  ((MIOV_PRETTY_PRINT)?   \
                MIO_out_make_text(-2,0,0,MIOC_P_max,"")\
		:MMMNULL)

/* MIOC_DONT_SPLIT-Bit auf special-Komponente ausblenden */
inline long
MIO_bitmask(long spec) {return(spec & ~(MIOC_DONT_SPLIT | MIOC_BADNESS));}

/* Wie MDB_make_debug_info, nur ist der Schluessel bereits bekannt und
   nicht der Filename */
#define MIO_make_debug_info( line, key, valid )   \
	( (line << SHIFT) | (key << 1) | valid )

#define MIO_create_info( line )		\
	MDB_create_debug_info( MIO_make_debug_info( line, MIOV_KEY, 1) )

/* Macros zur Erstellung eines tempor"aren Feldes von 'num'-char-Werten */

#define MIO_new_string(l, sp, num)                                    \
	{ sp = MMMmalloc(MMM_TMP,sizeof(char)*((num)+1), 0);\
          *MMMarr(&sp,(num),char) = '\0';                      \
	  l = MMMZ(&sp,char);                                \
	  MMMpoolinsert(&l); }


#define MIO_free_string(l, sp)                                        \
        { MMMpooldelete(&l);					      \
	  MMMfree(sp); }

/* Berechnung des Maximums zweier Zahlen            */

#define MIO_max(a,b) (((a)<(b))?(b):(a))

#ifdef MDX
#  define MIOC_RESTORE 0
#  define MIOC_SET  1
#  define MIOC_RESET 2

   void
   MIO_change_global(long);
#endif

/****************** Funktionen in den Dateien MIO_*.c **********************/



/**** nun folgen die Ausgabe-Funktionen fuer die Kategorien *****/

S_Pointer
MIO_out_cat_array(S_Pointer, long);

S_Pointer
MIO_out_cat_bool(S_Pointer, long);

#ifdef MDX
S_Pointer
MIO_out_cat_debug(S_Pointer,long);
#endif

S_Pointer
MIO_out_cat_dom(S_Pointer, long) ;

S_Pointer
MIO_out_cat_exec (S_Pointer, long) ;

S_Pointer
MIO_out_cat_ext(S_Pointer, long) ;

S_Pointer
MIO_out_cat_failed(S_Pointer, long);

S_Pointer
MIO_out_cat_var(S_Pointer, long);

S_Pointer
MIO_out_cat_proc_env(S_Pointer, long);

S_Pointer
MIO_out_cat_func_env(S_Pointer, long);

S_Pointer
MIO_out_cat_ident(S_Pointer, long);

S_Pointer
MIO_out_cat_namespace(S_Pointer, long);

S_Pointer
MIO_out_cat_nil(S_Pointer, long);

S_Pointer
MIO_out_cat_null(S_Pointer, long);

S_Pointer
MIO_out_cat_point(S_Pointer, long);

S_Pointer
MIO_out_cat_poly(S_Pointer, long);

S_Pointer
MIO_out_cat_polygon(S_Pointer, long);

S_Pointer
MIO_out_cat_proc(S_Pointer, long);

S_Pointer
MIO_out_cat_set(S_Pointer, long);

S_Pointer
MIO_out_cat_stat_list(S_Pointer, long);

S_Pointer
MIO_out_cat_string(S_Pointer, long);

S_Pointer
MIO_out_cat_table(S_Pointer, long);

S_Pointer /* fuer alle PARI-Zahl-Typen ausser CAT_COMPLEX */
MIO_print_number(S_Pointer, long);

S_Pointer /* fuer CAT_COMPLEX */
MIO_print_complex_number(S_Pointer, long);


/******************** interne Hilfsroutinen   ********************/
void
MIO_put_char(char);
S_Pointer 
MIO_append_operator(S_Pointer, S_Pointer);

/* Diese beiden Funktionen sollten nicht direkt zugreifbar sein */
void
MIO_print_string(const char *); /* C-String schreiben */

void
MIO_print_char(char);     /* einen char schreiben */

/* Formatierte Bildschirmausgabe der Zahl 'i'.				*/
/* Die Formatanweisung  's' darf genau ein Element besitzen.		*/
void
MIO_print_format_number(char *, long);

void
MIO_print_format_string(char *, char *);

/* Ausgabe eines Operator-Strings als MMM_STRING */
S_Pointer
MIO_print_symbol(S_Pointer);

/* Ausgabe eines MMM_STRING incl. Expandierung von Sonderzeichen */
S_Pointer
MIO_out_string(S_Pointer, bool maskBackquote = false);

/* Umwandlung einer Pari-Zahl in einen String */
S_Pointer
MIO_pari_num2str(S_Pointer);

/* Ausgabe einer Pari-Zahl */
void
MIO_pari_print_number(S_Pointer);

/* gibt die Argumente eines Funktionsaufrufs geklammert aus */
S_Pointer
MIO_out_args(S_Pointer *);

/* gibt die Argumente einer CAT_EXPR aus */
S_Pointer
MIO_out_args_expr(S_Pointer *,long,S_Pointer);

/* gibt alle Pointer eines S_Pointers als exprseq aus
   wird in MIO_out_cat_ext und MIO_out_cat_exec verwendet */
S_Pointer
MIO_out_all_args (S_Pointer, bool) ;

/* gibt die Argumente eines _index-Ausdrucks in eckigen Klammern aus */
S_Pointer
MIO_out_index_args(S_Pointer,long);

/* fuehrt die Evaluierung aller "print"-Methoden durch */
S_Pointer
MIO_call_eval(S_Pointer, S_Pointer);

/* Typpruefung und Ausgabe eines S_Pointers */
S_Pointer
MIO_out_help(S_Pointer *);

/* Verzweigung je nach Category */
S_Pointer
MIO_out_category(S_Pointer *,long,S_Pointer);

/* Verzweigung und Klammerung aller Nicht-Expressions */
S_Pointer
MIO_out_node(S_Pointer *,long,S_Pointer);

/* Verzweigung aller Expressions */
S_Pointer
MIO_out_expr(S_Pointer *,long,S_Pointer);

/* Zuruecksetzen des Ausgabepuffers */
void
MIO_newline();


/* BLOCK-Modus fuer Tables, Matrizen und Anweisungen */
void
MIO_init_BLOCK();
void
MIO_exit_BLOCK();


/* Pretty-Ausgabe spezieller Ausdruckstypen */
S_Pointer
MIO_out_stand_op_pretty(S_Pointer, S_Pointer, long, S_Pointer);
S_Pointer
MIO_out_plus(S_Pointer,S_Pointer,long,S_Pointer);
S_Pointer 
MIO_sort_sum(S_Pointer s);
S_Pointer
MIO_out_mult(S_Pointer,S_Pointer,long,S_Pointer);
S_Pointer
MIO_out_power(S_Pointer,S_Pointer,long,S_Pointer);
S_Pointer
MIO_out_stmtseq(S_Pointer,S_Pointer,long,S_Pointer);


/* Hilfsroutinen zur Ausgabe von Anweisungen */
short
MIO_out_stmt(S_Pointer *,long,S_Pointer);
S_Pointer
MIO_init_stmt();
void
MIO_exit_stmt();
void
MIO_print_blanks(long,bool);


/* Manipulation von MuPAD-Daten fuer die Pretty-Ausgabe */
bool
MIO_is_neg(S_Pointer);
S_Pointer
MIO_negate(S_Pointer, bool plusMode=false);

bool
MIO_has_normal_form(S_Pointer);

bool
MIO_is_inverted(S_Pointer);
S_Pointer
MIO_invert(S_Pointer);


/* Hilfsroutinen zur Erzeugung des Ausgabebaums und dessen Ausgabe */
S_Pointer
MIO_out_make_node(MIOT_FIELDSIZE, MIOT_FIELDSIZE, MIOT_FIELDSIZE,
		  MIOT_FIELDSIZE, long);
S_Pointer
MIO_out_make_text(MIOT_FIELDSIZE, MIOT_FIELDSIZE, MIOT_FIELDSIZE,
		  MIOT_FIELDSIZE, const char *);
S_Pointer
MIO_out_make_field(MIOT_FIELDSIZE);
void
MIO_out_write_in_field(S_Pointer , MIOT_FIELDSIZE , MIOT_FIELDSIZE *);
void
MIO_out_node_to_field(S_Pointer , MIOT_FIELDSIZE , MIOT_FIELDSIZE *);
void
MIO_out_text_to_field(S_Pointer , MIOT_FIELDSIZE , MIOT_FIELDSIZE *);

enum MIOT_centerTypes {MIOC_centered, MIOC_uncentered, MIOC_centered_nonl};
void
MIO_out_print_line(MIOT_FIELDSIZE, MIOT_centerTypes = MIOC_centered);

/* Erzeugung spezieller Strukturen */
S_Pointer
MIO_out_write_line(MIOT_FIELDSIZE);
S_Pointer
MIO_out_vertical_line(MIOT_FIELDSIZE);
S_Pointer
MIO_not(S_Pointer);
S_Pointer
MIO_div(S_Pointer);
S_Pointer
MIO_minus(S_Pointer, long, long);
S_Pointer
MIO_bracket(S_Pointer);
S_Pointer
MIO_out_brackets(S_Pointer, char);
S_Pointer
MIO_open_bracket(MIOT_FIELDSIZE ,char);
S_Pointer
MIO_close_bracket(MIOT_FIELDSIZE ,char);
S_Pointer
MIO_out_evaluate_children(S_Pointer ,long ,long , S_Pointer);
void
MIO_make_STRING_BUFFER(S_Pointer);
S_Pointer
MIO_flush_STRING_BUFFER();
void
MIO_make_NUM_BUFFER(S_Pointer);
S_Pointer
MIO_flush_NUM_BUFFER();
void
MIO_open_line();
void
MIO_close_line();
void
MIO_make_FIELD_BUFFER(S_Pointer);
S_Pointer
MIO_miostring2mmmstring(S_Pointer);


/* Routinen zur Bearbeitung des Ausgabebaums */
void
MIO_put_pointer(S_Pointer,S_Pointer,long);
void
MIO_get_param(S_Pointer,MIOT_FIELDSIZE *,MIOT_FIELDSIZE *,
	      MIOT_FIELDSIZE *,MIOT_FIELDSIZE *);
void
MIO_put_param(S_Pointer,MIOT_FIELDSIZE,MIOT_FIELDSIZE,
	      MIOT_FIELDSIZE,MIOT_FIELDSIZE spec);
MIOT_FIELDSIZE
MIO_get_priority(S_Pointer);
void
MIO_put_priority(S_Pointer, MIOT_FIELDSIZE);
void
MIO_get_pos(S_Pointer,long,MIOT_FIELDSIZE *line,MIOT_FIELDSIZE *);
void
MIO_put_pos(S_Pointer,long,MIOT_FIELDSIZE,MIOT_FIELDSIZE );



/* globale Variable in den Ausgabefiles  */

/* Steuerung der Zahlausgabe bei PRETTY_PRINT */
extern bool MIOV_PRINT_NUMBER;

/* Steuerung der Stringausgabe bei PRETTY_PRINT */
extern bool MIOV_PRINT_STRING;

/* Flag fuer die Art der Ausgabe bei PRETTY_PRINT */
extern bool MIOV_PRINT_BLOCK;

/* Flag fuer die Art der Ausgabe bei PRETTY_PRINT */
extern bool MIOV_PRINT_STMT;

/* Stringlaenge bei PRETTY_PRINT */
extern MIOT_FIELDSIZE MIOV_STRINGLEN;

/* steuert die Erzeugung von Debug-Info's */
extern bool  MIOV_DEBUG_MODE;

/* steuert die Erzeugung von Debug-Info's */
extern bool  MIOV_REALLY_DEBUG;

/* Aktuelle Zeilennummer fuer Debug-Mode */
extern long     MIOV_LINES;

/* Nummer der Ausgabedatei fuer Debug-Mode*/
extern long     MIOV_KEY;

/* wir erzeugen gerade eine expr2text-Ausgabe */
extern bool  MIOV_EXPR2TEXT_MODE;

#ifdef MDX
#  ifndef DOS_FILENAMES
#    include "MDB_debug.h"
#  else
#    include "MDB_debu.h"
#  endif


/* letzte Debug-Info */
extern DBX_TYPE MIOV_GLOBAL_DBX;

/* Rekursionstiefe der STMTSEQes */
extern long 	MIOV_NUM_STMTSEQ;

#endif  /* MDX */

/* wird fuer KeepOrder benoetigt, vgl. MIO_out_plus */
extern bool  MIOV_DOM_EXT_MODE;

/* Anzahl der Zeichen, die bisher in der aktuellen Zeile stehen */
extern long MIOV_ACT_LEN;

/* MMM-String mit Operator-Symbolen */
MTR_DECL_LITERAL(MIOV_S_PLUS)
MTR_DECL_LITERAL(MIOV_S_MINUS)
MTR_DECL_LITERAL(MIOV_S_MULT)
MTR_DECL_LITERAL(MIOV_S_DIVIDE)
MTR_DECL_LITERAL(MIOV_S_POWER)
MTR_DECL_LITERAL(MIOV_S_EXPRSEQ)
MTR_DECL_LITERAL(MIOV_S_STMTSEQ)

MTR_DECL_LITERAL(MIOV_DOMAIN_ASS_OUTPUT)    /* CAT_STRING "print" */
MTR_DECL_LITERAL(MIOV_DOMAIN_ASS_EXPR2TEXT) /* CAT_STRING "expr2text" */


/* Routinen des Files MIO_array.c */

void
MIO_out_array_head(long dim,long *start,long *range);

S_Pointer
MIO_out_undivided_array(S_Pointer    s,
			long         dim,
			long        *start,
			long        *range);


S_Pointer
MIO_out_undivided_array_pretty(	S_Pointer     s,
		 		long          dim,
	        		long         *start,
				long         *end,
		 		long         *actpos);

void
MIO_out_undivided_array_without_pretty(	S_Pointer  s,
		 		long          dim,
	        		long         *start,
				long         *end,
		 		long         *actpos);

S_Pointer
MIO_out_undivided_matrix(	S_Pointer     s,
		 		long          dim,
	        		long         *start,
				long         *end,
		 		long         *actpos);


#endif /* __MIO_local__ */
